package ueb04

import scalajs.js
import scalajs.js.annotation._
import org.scalajs.dom._

@JSExport
object Main extends js.JSApp {  
  def main = { 
    document.onreadystatechange = (e: Event) => {
      if (document.readyState == "interactive") init
    }
  }

  def init = {
    import upickle._

    val token = Option(localStorage.getItem("robotId"))

    val socketURL = (location.protocol match {
      case "http:" => "ws://"
      case "https:" => "wss://"
    }) + location.host + "/session?robot=" + token.getOrElse("")

    val socket = new WebSocket(socketURL)

    socket.onopen = { (e: Event) => 
      val deregisterInputLoop = inputLoop(c => socket.send(write(c)))
      socket.onclose = (e: Event) => {
        deregisterInputLoop()
        console.warn("connection to server lost... reloading")
        location.reload()
      }
    }

    socket.onmessage = { (e: MessageEvent) =>
      handleRobotStateUpdates(read[RobotStateUpdate](e.data.asInstanceOf[String]))
    }
  }

  def inputLoop(f: Control => Unit): (Unit => Unit) = {
    lazy val eventHandler: scalajs.js.Function1[Event,Unit] = { (e: Event) =>
      val event = e.asInstanceOf[KeyboardEvent]
      event.stopImmediatePropagation()
      Control.parseKeycode(event.keyCode).foreach { c =>
        console.info(s"[input]: $c")
        f(c)
      }
    }
    document.addEventListener("keydown", eventHandler)
    return (Unit=> document.removeEventListener("keydown", eventHandler))
  }

  def handleRobotStateUpdates: PartialFunction[RobotStateUpdate,Unit] = {
    case RobotId(id) => localStorage.setItem("robotId", id)
    case BatteryUpdate(level) => document.getElementById("status").innerHTML = "battery level: " + level
  }
}
