package ueb04

import akka.actor.{ActorLogging, Actor, ActorRef, Props}

/**
 * @author Martin Ring <martin.ring@dfki.de>
 * @date   08.06.15 14:20
 */
object Robot {
  def props(uniqueId: String) = Props(classOf[Robot], uniqueId)

  object Subscribe
  object Unsubscribe
}

class Robot(uniqueId: String) extends Actor with ActorLogging {
  var batteryLevel = 10

  var subscribers = Set.empty[ActorRef]

  def broadcastUpdate(update: RobotStateUpdate) = {
    subscribers.foreach(_ ! update)
  }

  def receive = {
    case Robot.Subscribe =>
      log.debug("new subscription from {}", sender)
      sender ! RobotId(uniqueId)
      sender ! BatteryUpdate(batteryLevel)
      subscribers += sender
    case Robot.Unsubscribe =>
      log.debug("{} unsubscribed", sender)
      subscribers -= sender
    case control: Control if batteryLevel > 0 =>
      log.debug("executing {}", control)
      batteryLevel -= 1
      broadcastUpdate(BatteryUpdate(batteryLevel))
    case control: Control =>
      log.info("received {} but battery is empty", control)
  }
}