package ueb04

import akka.actor._
import akka.stream._
import akka.http.scaladsl._
import akka.http.scaladsl.model.ws._
import akka.http.scaladsl.server._
import akka.stream.scaladsl._
import scala.util.{Success,Failure}

object Server extends App with Directives {
  implicit val system = ActorSystem("ueb04")  
  implicit val materializer = ActorFlowMaterializer()
  import system.dispatcher

  val robots = collection.mutable.Map.empty[String,ActorRef]

  def getRobot(id: String): ActorRef = robots.get(id) match {
    case Some(robot) => robot
    case None =>
      val id = java.util.UUID.randomUUID().toString
      val robot = system.actorOf(Robot.props(id))
      robots += id -> robot
      robot
  }

  def routes = get {
    pathSingleSlash(getFromResource("web/index.html")) ~    
    path("session"){
      parameter('robot)(robot => handleWebsocketMessages(websocketFlow(robot)))
    } ~
    path(PathMatchers.Rest)(path => getFromResource(path))
  } ~ getFromResourceDirectory("web")

  import upickle._

  def websocketFlow(robotId: String): Flow[Message, Message, Unit] =
    Flow[Message].collect { case TextMessage.Strict(msg) => upickle.read[Control](msg) }
      .via(robotIO(getRobot(robotId)))
      .map(msg => TextMessage.Strict(upickle.write(msg)))

  def robotIO(robot: ActorRef): Flow[Control,RobotStateUpdate,Unit] = {
    val source = Source.actorPublisher[RobotStateUpdate](WebBrowserOutput.props(robot))
    val sink = Sink.actorSubscriber[Control](WebBrowserInput.props(robot))
    Flow.wrap(sink, source){ case (_,_) => () }
  }
  
  val binding = Http(system).bindAndHandle(routes, "localhost", 8080)
  
  binding.onComplete {
    case Success(binding) =>
      val localAddress = binding.localAddress
      println(s"Server is listening on ${localAddress.getHostName}:${localAddress.getPort}")
      readLine()
      system.shutdown()
    case Failure(e) =>
      println(s"Binding failed with ${e.getMessage}")
      system.shutdown()
  }
}