package ueb04

import akka.actor.{ActorLogging, Props, ActorRef}
import akka.stream.actor.ActorSubscriberMessage.{OnError, OnComplete, OnNext}
import akka.stream.actor._

object WebBrowserOutput {
  def props(robot: ActorRef) = Props(classOf[WebBrowserOutput], robot)
}

class WebBrowserOutput(robot: ActorRef) extends ActorPublisher[RobotStateUpdate] {
  def receive = unsubscribed

  def unsubscribed: Receive = {
    case ActorPublisherMessage.Request(n) =>
      robot ! Robot.Subscribe
      context.become(subscribed)
  }

  def subscribed: Receive = {
    case ActorPublisherMessage.Cancel =>
      robot ! Robot.Unsubscribe
      context.become(unsubscribed)
    case update: RobotStateUpdate if (totalDemand > 0) =>
      onNext(update)
  }
}

object WebBrowserInput {
  def props(robot: ActorRef) = Props(classOf[WebBrowserInput], robot)
}

class WebBrowserInput(robot: ActorRef) extends ActorSubscriber with ActorLogging {
  def receive = {
    case OnNext(control: Control) =>
      robot ! control
    case OnComplete =>
      log.debug("web browser input diconnected")
    case OnError(e) =>
      log.error(e, "web browser inpud terminated abnormally with {}", e.getMessage)
  }

  override protected def requestStrategy: RequestStrategy = OneByOneRequestStrategy
}